import 'dart:async';

import 'package:dot_nfc_platform_interface/dot_nfc_platform_interface.dart';
import 'package:flutter/widgets.dart';

/// Called when the NFC data is read successfully.
/// Once this callback is called, the whole process is considered to be over.
typedef OnNfcTravelDocumentReaderSucceededCallback = void Function(
    TravelDocument travelDocument);

/// Called upon user cancellation through the iOS system dialog's cancel button.
/// Once this callback is called, the whole process is considered to be over.
typedef OnNfcTravelDocumentReaderCancelledCallback = void Function();

/// Called when the process encountered an error.
/// Once this callback is called, the whole process is considered to be over.
typedef OnNfcTravelDocumentReaderFailedCallback = void Function(
    NfcTravelDocumentReaderException exception);

/// NFC Travel Document Reader UI component (widget).
///
/// The widget for reading NFC enabled travel documents.
///
/// ## Widget configuration
///
/// In order to configure the behaviour of [NfcTravelDocumentReaderWidget], use [NfcTravelDocumentReaderConfiguration] and pass in to [NfcTravelDocumentReaderWidget.new]. This is a required parameter.
///
/// ## Control the widget and consume outputs
///
/// As soon as the widget is created, the NFC Travel Document Reader process starts.
///
/// When the NFC Travel Document Reader process finishes successfully, the result will be returned via the [onSucceeded] callback.
///
/// When the iOS NFC Travel Document Reader process is cancelled via pressing the cancel button of a system dialog, the result will be returned via the [onCancelled] callback.
///
/// When the NFC Travel Document Reader process encounters an error., the exception will be returned via the [onFailed] callback.
final class NfcTravelDocumentReaderWidget extends StatefulWidget {
  /// Configuration of the NFC Travel Document Reader process.
  final NfcTravelDocumentReaderConfiguration configuration;

  /// Called when the NFC data is read successfully.
  /// Once this callback is called, the whole process is considered to be over.
  final OnNfcTravelDocumentReaderSucceededCallback? onSucceeded;

  /// Called only when the iOS NFC process is cancelled.
  /// Once this callback is called, the whole process is considered to be over.
  final OnNfcTravelDocumentReaderCancelledCallback? onCancelled;

  /// Called when the process encountered an error.
  /// Once this callback is called, the whole process is considered to be over.
  final OnNfcTravelDocumentReaderFailedCallback? onFailed;

  NfcTravelDocumentReaderWidget({
    required this.configuration,
    this.onSucceeded,
    this.onCancelled,
    this.onFailed,
  });

  @override
  State createState() => _NfcTravelDocumentReaderWidgetState();
}

class _NfcTravelDocumentReaderWidgetState
    extends State<NfcTravelDocumentReaderWidget> {
  late NfcTravelDocumentReaderComponent _component;
  final List<StreamSubscription<void>> _subscriptions = [];

  @override
  void initState() {
    super.initState();

    _component =
        DotNfcPlatform.instance.createNfcTravelDocumentReaderComponent();

    _subscriptions.add(_component.onSucceeded().listen((event) {
      widget.onSucceeded?.call(event);
    }));

    _subscriptions.add(_component.onCancelled().listen((event) {
      widget.onCancelled?.call();
    }));

    _subscriptions.add(_component.onFailed().listen((event) {
      widget.onFailed?.call(event);
    }));
  }

  @override
  Widget build(BuildContext context) {
    return _component.buildView(widget.configuration);
  }

  @override
  void dispose() {
    for (var subscription in _subscriptions) {
      subscription.cancel();
    }
    _component.dispose();
    super.dispose();
  }
}
